;;;
;;; $Id: manued.el,v 1.21 2002/08/12 15:39:29 yamauchi Exp $
;;;
;;; manued.el --- minor mode for manued proofreading method.
;;;
;;; Manued stands for MANUscripting EDitor.
;;; (This program is my first E-lisp program.)
;;;
;;; Author:     YAMAUCHI, Hitoshi $B;3Fb(B $B@F(B <yamauchi@mpi-sb.mpg.de>
;;; Maintainer: YAMAUCHI, Hitoshi $B;3Fb(B $B@F(B <yamauchi@mpi-sb.mpg.de>
;;; Updated: 2000-10-20
;;; Version: 0.9.4-current
;;; Keywords: proofreading, docs
;;;
;;;
;;; Original idea of manued : 
;;;	TAKEUCHI Ikuo (nue@nue.org), ``Manuscripting Editing on
;;;	E-mail,'' 39th Programming Symposium, 1998, January, pp.61--68   
;;;
;;;	The original paper is written in Japanese,
;;;	$BC]Fb0jM:(B, ``$BEE;R%a!<%k$G869F$r=$@5$9$kJ}K!(B --- Manuscript
;;;     Editing (Manued, $B??s,F;(B)$B$rL\;X$7$F(B ---'', $BBh(B 39 $B2s%W%m%0%i%_%s(B 
;;;	$B%0%7%s%]%8%&%`(B, 1998, 1$B7n(B, pp.61--68
;;;	
;;; Lisp tutor : 
;;;	MAEDA Atusi (maeda@is.uec.ac.jp) I very thank to MAEDA Atusi.
;;;	He taught me Lisp and the Programming Symposium. If I
;;;	didn't met him, this program is never written.
;;;
;;;
;;; Copyright Notice:
;;;
;;; Copyright (C) 1998--2000 YAMAUCHI, Hitoshi $B;3Fb(B $B@F(B <yamauchi@acm.org>
;;; All rights reserved.
;;;
;;; This program is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either versions 2, or (at your option)
;;; any later version.
;;;
;;; This program is distributed in the hope that it will be useful
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; Contributors : Thanks a lot.
;;;	Stefan Monnier  (0.9.1)
;;;	NAKAJIMA, Mikio (0.9.3)
;;;	KAWAMURA, Takao (0.9.3)
;;;	
;;------------------------------------------------------------
;; debug $BMQ$N(B message $B=PNO(B
;;------------------------------------------------------------
;; delete at release
;;(setq debug-on-error t)
;;(defun dbg (mes) (print mes (get-buffer "manued-debug")))

;;------------------------------------------------------------
;; constant values
;;------------------------------------------------------------
(defconst manued-version-num   "0.9.5-current" 
  "The version of manued.el.
$B??s,F;$N%P!<%8%g%s(B")

(defconst manued-build-day "2002-8-12"
  "The day of last change of manued.el.
$B??s,F;$N:G=*99?7F|(B")

(defconst manued-formatted-buffer-name "*manued*"
  "Buffer name of formatted manued text.
$B??s,F;$N@07A:Q$_%P%C%U%!L>(B")

;;------------------------------
;; What kind of emacs is this?
;;------------------------------
(defconst manued-xemacs-p
  (and (featurep 'mule) (string-match "XEmacs" emacs-version))
  "Non-nil when running on XEmacs.")

;;;------------------------------------------------------------
;;; some useful macros
;;;------------------------------------------------------------

;;------------------------------------------------------------
;; convenient funcs for pstr (points with string region)
;;	pstr = (command-begin-point command-end-point command-str)
;;------------------------------------------------------------
(defmacro manued-get-first-point (pstr) 
  "pstr $B$+$i%3%^%s%I$N=i$a$N(B point $B$rJV$9(B"
  `(car ,pstr))

(defmacro manued-get-end-point (pstr) 
  "pstr $B$+$i%3%^%s%I$N=*$o$j$N(B point $B$rJV$9(B"
  `(car (cdr ,pstr)))

(defmacro manued-get-command-str (pstr) 
  "pstr $B$+$i%3%^%s%IJ8;zNs$rJV$9(B"
  `(car (cdr (cdr ,pstr))))

(defmacro manued-command-eq (pstr command-chars)
  "pstr $BCf$N%3%^%s%IJ8;zNs$,%3%^%s%IJ8;zNs(B(command-chars)$B$H0lCW$9$k>l(B
$B9g$K(B t $B$rJV$9(B"   
  `(string-equal 
    (manued-get-command-str ,pstr) ,command-chars))

;;;------------------------------------------------------------
;;; manued version
;;;------------------------------------------------------------
(defun manued-show-version ()
  "Print manued version.
manued $B$N%P!<%8%g%s$rCN$i$;$k(B."
  (interactive)
  (cond ((interactive-p)
         (message "Manued version %s of %s" 
		  manued-version-num manued-build-day))))

;;;
;;; Variable declarations
;;;
;;------------------------------
;; related manued command variables
;;------------------------------
(defvar manued-l-parenthesis-str "["
  "* Start string of manued command. : default is `['
$B??s,F;$N%3%^%s%I$N$O$8$^$j$r<($9J8;zNs(B")
(make-variable-buffer-local 'manued-l-parenthesis-str)

(defvar manued-r-parenthesis-str "]"
  "* End string of manued command. : default is `]'
$B??s,F;$N%3%^%s%I$N=*N;$r<($9J8;zNs(B")
(make-variable-buffer-local 'manued-r-parenthesis-str)

(defvar manued-swap-str "|"
  "* String of manued swap-command. : default is '|'
	'A|B|C' means to swap A with C, then A|B|C will be C|B|A. 
	Especially, 'A||C' means to swap A and C, then A||C will be C||A.
$B8r49%3%^%s%IJ8;zNs(B : $B%G%U%)%k%H$O(B '|':
	A|B|C $B$J$i$P(B A $B$H(B C $B$rF~$l49$($k!%$7$?$,$C$F(B A|B|C $B$O(B C|B|A $B$H$J$k!%(B
	$BFC$K(B 'A||C' $B$O(B A $B$H(B C $B$NF~$l49$($r0UL#$9$k!%$7$?$,$C$F!$(B
	A||C $B$O(B C||A $B$H$J$k!%(B")
(make-variable-buffer-local 'manued-swap-str)

(defvar manued-delete-str "/"
  "* String of manued delete-command . : default is '/'
	'A/B' means to substitute A by B, then A/B will be B. 
	Especially, '/B' means to insert B and 'A/' means to delete A.
$B>C5n%3%^%s%IJ8;zNs(B $B%G%U%)%k%H$O(B '/':
	A/B $B$J$i$P(B A $B$r(B B $B$GCV$-49$($k!%(B
	$BFC$K(B '/B' $B$O(B B $B$NA^F~$r0UL#$7!$(B'A/' $B$O(B A $B$N:o=|$r0UL#$9$k(B.")
(make-variable-buffer-local 'manued-delete-str)

(defvar manued-comment-str ";"
  "* String of maenud comment command. : default is `;'
comment out until manued-r-parenthesis-str. 
$B%3%a%s%HJ8;z!%(B $B%G%U%)%k%H$O(B ';':
	$B??s,F;%3%^%s%I$N=*$j$^$G$r%3%a%s%H$H$_$J$9!%(B")
(make-variable-buffer-local 'manued-comment-str)

(defvar manued-escape-str "~"
  "* Escape string. : default is `~'
This string can escape a next adjacent manued command. 
$B%(%9%1!<%WJ8;z!%%G%U%)%k%H$O(B '~':
	$B<!$N??s,F;%3%^%s%IJ8;z$r%(%9%1!<%W$9$k!%(B")
(make-variable-buffer-local 'manued-escape-str)

(defvar manued-pretty-print-format-delete-list 
  '("\\textnormal{%s}" "\\textbf{%s}" "\\textit{%s}")
  "* manued pretty print format strings list for delete command.
'(delete-part-format replaced-part-format comment-part-format)
as the default, [A/B;C] will be print out:
  \\textnormal{A}\\textbf{B}\\textit{C}
$B>C5n%3%^%s%IMQ$N(B pretty print $B$N:]$KMxMQ$5$l$k(B format $B$N%j%9%H(B
$B%G%U%)%k%H$G$O!$(B[A/B;C] $B$O<!$N$h$&$K=PNO$5$l$k(B:
  \\textnormal{A}\\textbf{B}\\textit{C}")
(make-variable-buffer-local 'manued-pretty-print-format-delete-list)

(defvar manued-pretty-print-format-swap-list 
  '("\\textbf{%s}---" "\\textbf{%s}" "---\\textbf{%s}" "(\\textit{%s})")
  "* manued pretty print format strings list for swap command.
'(alpha-part-format beta-part-format gamma-part-format comment-part-format)
as the default, [A|B|C;D] will be print out:
 \\textbf{%s}---\\textbf{%s}---\\textbf{%s}(\\textit{%s})
$B8r49%3%^%s%IMQ$N(B pretty print $B$N:]$KMxMQ$5$l$k(B format $B$N%j%9%H(B
$B%G%U%)%k%H$G$O!$(B[A|B|C;D] $B$O<!$N$h$&$K=PNO$5$l$k(B:
 \\textbf{%s}---\\textbf{%s}---\\textbf{%s}(\\textit{%s})
")
(make-variable-buffer-local 'manued-pretty-print-format-swap-list)

(defvar manued-pretty-print-on-p nil
  "* manued pretty print on.
If this is t, you will get revised/original document with pretty
print style. The style will be changed by variables
manued-pretty-print-format-swap-list and
manued-pretty-print-format-delete-list. 
pretty print $B$r(B off $B$K$9$k!%(B
$B$b$7$3$NCM$,(B t $B$N>l9g$K$O!$J8=q$r@07A$7$?>l9g$K(B pretty print style $B$G(B
$B=PNO$5$l$^$9!%$=$N%9%?%$%k$OJQ?t(B manued-pretty-print-format-swap-list $B$H(B
manued-pretty-print-format-delete-list $B$G@)8f$5$l$^$9!%(B")
(make-variable-buffer-local 'manued-pretty-print-on-p)

(defvar manued-pretty-print-null-comment-out-p nil
  "* When t and there is no comment part, comment is considered as
\"\" and output comment part in pretty print mode. When nil and there
is no comment part, no output for comment part. Notice. This is not
buffer local variable. 
$B$3$NCM$,(B t $B$N>l9g$G$+$D%3%a%s%H$,L5$$>l9g!$6u$N%3%a%s%H$,$"$k$H9M$($F(B
format $B$N0z?t$K(B \"\" $B$,EO$5$l!$=PNO$5$l$^$9!%$b$7$3$NJQ?t$,(B nil $B$N>l9g$G(B
$B%3%a%s%HItJ,$,B8:_$7$J$$>l9g$K$O!$%3%a%s%HItJ,$O=PNO$5$l$^$;$s!%(B
$B$3$NJQ?t$O(B buffer local $B$G$O$"$j$^$;$s!%(B")

(defconst manued-defversion-str manued-version-num
  "* String of manued version number. : default is same as version
number of this code.   
$B%P!<%8%g%sHV9f$r<($9J8;z(B : $B%G%U%)%k%H$O$3$N%3!<%I$N%P!<%8%g%sHV9f(B")

;;------------------------------
;; related `Find and set def* pattern'.
;;------------------------------
(defvar manued-doc-begin-pat "-*-*- BEGINMANUED -*-*-"
  "* This pattern indicates the beginning of a manued document. 
Default is `-*-*- BEGINMANUED -*-*-'. There is no such pattern in
the document, start point of the manued document is set to `point-min'. 
When this variable is nil, beginning point is always `point-min'.

$B??s,F;$NJ8=q$N:G=i$r<($9J8;zNs!%$3$NJ8;zNs$,J8=qCf$KB8:_$7$J$$>l9g$K$O(B 
`point-min' $B$,@hF,$H$7$FMQ$$$i$l$k!%$^$?!$$3$N%7%s%\%k$,(B nil $B$N>l9g$K(B
$B$O>o$K(B `point-min' $B$,??s,F;J8=q$N;O$^$j$H$_$J$5$l$k!%(B")
(make-variable-buffer-local 'manued-doc-begin-pat)

(defvar manued-doc-end-pat "-*-*- ENDMANUED -*-*-"
  "* This pattern indicates end of a manued document. 
default is `-*-*- ENDMANUED -*-*-'. There is no such pattern in
the document, enx d point of the manued document is set to `point-max'. 
When this variable is nil, beginning point is always `point-max'.

$B??s,F;$NJ8=q$N:G8e$r<($9J8;zNs!%$3$NJ8;zNs$,J8=qCf$KB8:_$7$J$$>l9g$K$O(B 
`point-max' $B$,:G8e$H$7$FMQ$$$i$l$k!%$^$?!$$3$N%7%s%\%k$,(B nil $B$N>l9g$K(B
$B$O>o$K(B `point-max' $B$,??s,F;J8=q$N;O$^$j$H$_$J$5$l$k!%(B")
(make-variable-buffer-local 'manued-doc-end-pat)

(defconst manued-def-alist
  '(("defLparenthesis"	manued-l-parenthesis-str)
    ("defRparenthesis" 	manued-r-parenthesis-str)
    ("defswap"		manued-swap-str)
    ("defdelete"	manued-delete-str)
    ("defcomment"	manued-comment-str)
    ("defescape"	manued-escape-str)
    ("deforder"		manued-order-str)
    ("defversion"	manued-defversion-str))
  "manued command definition strings and symbols
$B??s,F;$N%3%^%s%I@k8@Dj5AJ8;zNs$H$=$N%7%s%\%k(B")

(defvar manued-defcommand-head-str-list 
  '("%" "%%"				; for TeX
    )
  "list of manued defcommand header strings.  
This strings are ignored when they added at head of manued command
definition string.

There is some cases that we want to comment out the manued
defcommands, for example using manued in TeX document. We can write
defcommands like below.

	%%defparentheses [ ]
	%%defdelete	 /

These defcommands are recognized by manued.el and TeX ignore these
commands.

$B$3$NJ8;zNs$N%j%9%H$K$O!$??s,F;$N%3%^%s%I@k8@Dj5AJ8;zNs$NA0$KIU2C$7$?>l(B
$B9g$KL5;k$9$kJ8;zNs$r4^$a$k!%$?$H$($P(B TeX $B$NJ8=qCf$G??s,F;$rMxMQ$9$k>l(B
$B9g!$0J2<$N$h$&$K??s,F;Dj5A%3%^%s%I$r(B TeX $B$N%3%a%s%HIt$K4^$a$k(B

	%%defparentheses [ ]
	%%defdelete	 /

$B$3$&$9$k$H!$(Bmanued.el $B$O$3$l$r(B defcommand $B$H$7$FG'<1$7!$$5$i$K(B TeX $B$G(B
$B$O$3$NItJ,$rL5;k$9$k$N$G!$F)2aE*$K(B manued $B$rMxMQ2DG=$G$"$k!%(B")
(make-variable-buffer-local 'manued-defcommand-head-str-list)

;;--------------------
;; hilit related defvar
;;--------------------
(defvar manued-use-color-hilit 'follow-font-lock-mode
  "t when using color hilit. However `window-system' is nil, this
value is set to nil. When this is 'follow-font-lock-mode, follow font
lock mode.  

$B?'$r;H$C$F%O%$%i%$%H$9$k>l9g$K$O(B t$B!$$?$@$7!$(Bwindow-system $B$,(B nil $B$N>l(B
$B9g$K$O(B nil $B$,%;%C%H$5$l$k!%(B'follow-font-lock-mode $B$N>l9g$K$O(B
font-lock-mode$B$K=>$&!%(B")
(make-variable-buffer-local 'manued-use-color-hilit)

;; hilit color
(defvar manued-first-hilit-color-list
  '("red"				; delete first color
    "gray60"				; delete last color
    "blue"				; swap alpha color
    "red"				; swap beta  color
    "green4"				; swap gamma color
    "BlueViolet"			; comment color 
    "gray60")				; command color
  "* hilit for first part of command.")
(make-variable-buffer-local 'manued-first-hilit-color-list)

(defvar manued-last-hilit-color-list
  '("gray60"				; delete first color
    "red"				; delete last color
    "blue"				; swap alpha color
    "red"				; swap beta  color
    "green4"				; swap gamma color
    "BlueViolet"			; comment color 
    "gray60")				; command color
  "* hilit for last part of command")
(make-variable-buffer-local 'manued-last-hilit-color-list)

;;------------------------------
;; insert command related variables
;;------------------------------
(defvar manued-is-delete-command-with-comment-on t 
  "Control insert-delete-command inserts one comment command or not.

When manued-is-delete-command-with-comment-on is t.
	manued-insert-delete-command                ... insert comment
	manued-insert-delete-command-toggle-comment ... not insert comment
When manued-is-delete-command-with-comment-on is nil.
	manued-insert-delete-command                ... not insert comment
	manued-insert-delete-command-toggle-comment ... insert comment

$B??s,F;(B command $BFbIt$G%3%a%s%HJ8;z$rA^F~$9$k$+$7$J$$$+$r@)8f$9$k!%(Bt $B$N(B
$B;~$K$O(B default $B$G(B manued-insert-delete-command $B$O%3%a%s%HJ8;z$rA^F~$9$k!%$?(B
$B$@$7!$(Bmanued-insert-delete-command-toggle-comment $B%3%^%s%I$O5U$NF0:n$r$9$k!%(B
")
(make-variable-buffer-local 'manued-is-delete-command-with-comment-on)

(defvar manued-is-swap-command-with-comment-on t 
  "Control insert-swap-command inserts one comment command or not.

When manued-is-swap-command-with-comment-on is t.
	manued-insert-swap-command                ... insert comment
	manued-insert-swap-command-toggle-comment ... not insert comment
When manued-is-swap-command-with-comment-on is nil.
	manued-insert-swap-command                ... not insert comment
	manued-insert-swap-command-toggle-comment ... insert comment

$B??s,F;(B command $BFbIt$G%3%a%s%HJ8;z$rA^F~$9$k$+$7$J$$$+$r@)8f$9$k!%(Bt $B$N(B
$B;~$K$O(B default $B$G(B manued-insert-swap-command $B$O%3%a%s%HJ8;z$rA^F~$9$k!%$?(B
$B$@$7!$(Bmanued-insert-swap-command-toggle-comment $B%3%^%s%I$O5U$NF0:n$r$9$k!%(B
")
(make-variable-buffer-local 'manued-is-swap-command-with-comment-on)


;;============================================================
;; Find and set def* pattern.
;;============================================================
;;
;; def* $B$N8e$m$KB3$/Hs6uGrJ8;z2?J8;z$+$r$=$N(B def $B$N%Q%?!<%s$H$9$k(B
;; $B8=:_$O0J2<$N$b$N$r%5%]!<%H$9$k!%8e$m$N$b$N$O(B default $BCM(B
;;
;; 	defparentheses  [ ]
;; 	; defLparenthesis [	(will be obsoleted but now support)
;; 	; defRparenthesis ]	(will be obsoleted but now support)
;; 	defswap 	|
;; 	defdelete 	/
;; 	defcomment 	;
;;	defescape  	~
;;	defversion  	manued-version-num
;;
;;
;; $B:G=i$O0LCV$N%-%c%C%7%e$O$7$J$$$3$H$K$9$k(B
;; $B<!$OJT=8$5$l$?$+$I$&$+$r8+$k$h$&$K$9$k(B
;;
;(defvar manued-doc-begin-point nil
;  "The cache of the beginnig point of manued document.
;   $B??s,F;J8=q$N%9%?!<%H%]%$%s%H$N%-%c%C%7%e(B")
;(defvar manued-doc-end-point nil
;  "The cache of the end point of manued document.
;  $B??s,F;J8=q$N%(%s%I%]%$%s%H$N%-%c%C%7%e(B")

;;
;; $B??s,F;J8=q$N$O$8$a$rC5$9(B
;;
(defun manued-get-doc-begin-point ()
  "Find beggining point of the manued document.
Find beginning pattern of a manued document and return the point. If
begging pattern is not founded in the manued document, return the
`point-min'. The beginning pattern is `manued-doc-begin-pat'.

$B??s,F;J8=q$N:G=i$N0LCV$rC5$7!$$=$N%]%$%s%H$rJV$9!%$b$7??s,F;J8=qCf$K??(B
$Bs,F;J8=q3+;O%Q%?!<%s$,$_$D$+$i$J$$>l9g$K$OJ8=q$N:G=i(B `point-min' $B$XHt(B
$B$V!%??s,F;J8=q3+;O%Q%?!<%s$O(B `manued-doc-begin-pat' $B$,J];}$7$F$$$k!%(B"
  (if (null manued-doc-begin-pat)
      (point-min)
    (save-excursion
      (goto-char (point-min))
      (if (search-forward manued-doc-begin-pat nil t)
	  (match-beginning 0)		; found
	(point-min)))))			; not found

;;
;; $B??s,F;J8=q$N=*$o$j$rC5$9(B
;;
(defun manued-get-doc-end-point ()
  "Find end point of the manued document.
Find end pattern of a manued document and return the point. If
end pattern is not founded in the manued document, return the
`point-max'. The end pattern is `manued-doc-end-pat'.

$B??s,F;J8=q$N=*$o$j$N0LCV$rC5$7!$$=$N%]%$%s%H$rJV$9!%$b$7??s,F;J8=qCf$K(B
$B??s,F;J8=q3+;O%Q%?!<%s$,$_$D$+$i$J$$>l9g$K$OJ8=q$N:G=i(B `point-min' $B$X(B
$BHt$V!%??s,F;J8=q3+;O%Q%?!<%s$O(B `manued-doc-begin-pat' $B$,J];}$7$F$$$k!%(B
"
  (if (null manued-doc-end-pat)
      (point-max)
    (save-excursion
      (goto-char (point-max))
      (if (search-backward manued-doc-end-pat nil t)
	  (match-end 0)			; found
	(point-max)))))			; not found

;;;------------------------------
;;; defcommand $B$rC5$7$F%;%C%H$9$k(B
;;;------------------------------
;; $B0l0z?t$N0l$D$N(B defcommand $B$rC5$7$F%;%C%H$9$k(B
(defun manued-search-set-def-one (decstr-str)
  "Find a manued command definition string `decstr-str' and set manued
command pattern. 
See also the `manued-def-alist' which is a list of manued command
declarations and variables.

$B0l$D$N??s,F;%3%^%s%I@k8@J8;zNs(B `decstr-str' $B$rC5$7$F$=$NJ8;zNs$KBP1~$9(B
$B$kJ8;zNs$r%;%C%H$9$k!%(B 
`manued-def-alist' $B$,??s,F;%3%^%s%I@k8@J8;zNs$H$=$NJQ?t$rJ];}$7$F$$$k(B
$B$N$G;2>H$N$3$H!%(B"
  (let ((non-whitespace-pat "[ |\t]+\\([^ |\t|$\n]+\\)"))
    (if (re-search-forward 
	 (concat "^" (car decstr-str) non-whitespace-pat) nil t)  
	(progn 
	  (let ((b (match-beginning 1))	(e (match-end 1)))
	    (if (< b e) 
		(set (car (cdr decstr-str))
		     (buffer-substring-no-properties b e))))))))

;; $BFs0z?t$N0l$D$N(B defcommand $B$rC5$7$F%;%C%H$9$k(B
(defun manued-search-set-defparentheses-with-comment (comment-str)
  "Find defparentheses and set values. This method is for
defparentheses only.

defparentheses $B$rC5$7!$CM$r%;%C%H$9$k!%(Bcomment-str $B$r@hF,$KIU2C$9$kJ8(B
$B;zNs!%(B"
  (if (re-search-forward
       (concat "^" comment-str "defparentheses[ |\t]+") nil t)
      ;; $B6uGr$G$J$$(B2$B$D$N0z?t$rC5$9!%$3$N(Bregex$B$rJ#?t=E$M$F$$$1$P;XDj(B
      ;; $B8D$N0z?t$r07$&$3$H$K$O$J$k(B
      (if (looking-at "\\([^ |\t|$\n]+\\)[ |\t]+\\([^ |\t|$\n]+\\)")
	  (progn
	    (setq manued-l-parenthesis-str
		  (buffer-substring-no-properties
		   (match-beginning 1) (match-end 1)))
	    (setq manued-r-parenthesis-str
		  (buffer-substring-no-properties
		   (match-beginning 2) (match-end 2))))
	(error (format ":Two arguments are needed for defparentheses.")) 
	nil)
    nil))

;;
;; $BA4$F$N(B defcommand $B$r%;%C%H$9$k(B
;;
(defvar manued-header-is-found nil)	; header $B$,??s,F;J8=qCf$K$"$k$+(B
(defun manued-search-set-defcommands ()
  "find all manued command declarations in a dcument.
$BA4$F$N??s,F;%3%^%s%I@k8@J8;zNs$rJ8=qCf$+$iC5$7=P$7!$??s,F;%3%^%s%I$r%;%C(B
$B%H$9$k!%(B
See also `manued-search-set-oneargdefs'."
  ;; header$B$r:F$SC5$9(B
  (setq manued-header-is-found nil)
  ;; defparentheses $B$rC5$7%;%C%H$9$k(B : 
  (manued-search-set-defparentheses)
  ;; $B0l0z?t$N(B def* $B$rC5$7%;%C%H$9$k(B
  (manued-search-set-oneargdefs manued-def-alist))

;;
;; defparentheses $B$rC5$7$=$NCM$r%;%C%H$9$k(B : 
;;  $B$3$l$@$1(Bdefcommand$B$NCf$G(B2$B0z?t$J$N$GJL$K$9$k!%B>$KJ#?t0z?t$N$b$N$,(B
;;  $B=P$F$/$l$PA4$F$N(Bdefcommand$B$OJ#?t0z?t$H$7$F0lHL2=$7$FBP=h$9$Y$-$@$,!$(B
;;  $B:#$O$3$l$7$+$J$$$N$GB?>/(Badd-hoc$B$G$O$"$k$,!$$3$N$h$&$KBP=h$7$?!%(B
;;
(defun manued-search-set-defparentheses ()
  (goto-char (manued-get-doc-begin-point)) ; goto begin
  (if (manued-search-set-defparentheses-with-comment "") ; serach and set one
      (setq manued-header-is-found t)       ; $BAG$N(B defparentheses $B$rH/8+(B
    ;; defparentheses $B$,AG$GB8:_$7$J$$>l9g%3%a%s%HJ8;z$r2C$($F$_$k(B
    (let ((comment-head-list manued-defcommand-head-str-list))
      (while comment-head-list
	(if (manued-search-set-defparentheses-with-comment
	     (car comment-head-list))
	    (progn		; $B%3%a%s%H(B + $B%3%^%s%I$,B8:_$7$?(B
	      (setq comment-head-list nil) 
	      (setq manued-header-is-found t))
	  (setq comment-head-list (cdr comment-head-list)))))))

;;
;; manued-doc-begin-pat $B$+$i=i$a$F(B manued-def-alist $BCf$N%Q%?!<%s$K9gCW(B
;; $B$9$kJ8;zNs$rC5$7$=$N0z?t$r%;%C%H$9$k!%0l0z?t$N(Bdefcommand$B$N$_$r07$&!%(B
;;
(defun manued-search-set-oneargdefs (def-alist)     
  "find one argument manued command declarations in a dcument.
$B0l0z?t$N??s,F;%3%^%s%I@k8@J8;zNs$rJ8=qCf$+$iC5$7=P$7!$??s,F;%3%^%s%I$r(B
$B%;%C%H$9$k!%(B
See also `manued-search-set-def-one'."
  (while def-alist
    (goto-char (manued-get-doc-begin-point)) ; goto begin
    (let ((defcom (car def-alist)))
      (if (manued-search-set-def-one defcom) ; serach and set one
	  (setq manued-header-is-found t)   ; $BAG$N(B defcommand $B$rH/8+(B
	;; defcommand $B$,AG$GB8:_$7$J$$>l9g%3%a%s%HJ8;z$r2C$($F$_$k(B
	(let ((comment-head-list manued-defcommand-head-str-list))
	  (while comment-head-list 
	    (if (manued-search-set-def-one 
		 (cons (concat (car comment-head-list) (car defcom)) 
		       (cdr defcom)))
		(progn		; $B%3%a%s%H(B + $B%3%^%s%I$,B8:_$7$?(B
		  (setq comment-head-list nil) 
		  (setq manued-header-is-found t))
	      (setq comment-head-list (cdr comment-head-list)))))
	;; else list $B$NMWAG$,$"$k$@$17+$jJV$9(B
	))
    (setq def-alist (cdr def-alist)))
  ;; $B8E$$%P!<%8%g%s$N$b$N$H8_49$r;}$?$;$k$?$a$N:n6H$r9T$&(B
  (manued-dispatch-for-old-version)    
  ;; defcommand $B$N0l4S@-$OJ]$?$l$F$$$k$+(B
  (manued-check-defcommand-consistency)
  ;; $BA4$F(B defcommand $B$r%;%C%H$7$?$i(B order $B$r7h$a$k(B (deforder $B$rH?1G$5$;$k(B)
  (manued-set-order-from-order-str)
  manued-header-is-found)

;; 
;; It is error when the same strings are used in defcommand.
;;	ex. Error if l-parenthesis-str and r-parenthesis-str are the
;;	same string.
;;	
(defun manued-check-defcommand-consistency ()
  (let ((compared-list manued-def-alist))
    (while (not (null compared-list))
      (let* ((compared-def (car compared-list))
	     (target-list (cdr compared-list))
	     (compared-str nil)
	     (target-str   nil))
	(while (not (null target-list))
	  (let* ((target-def (car target-list))
		 (s0 (symbol-value (car (cdr compared-def))))
		 (s1 (symbol-value (car (cdr target-def)))))

	    (if (<= (length s0) (length s1))
		(progn
		  (setq compared-str s0)
		  (setq target-str (substring s1 0 (length s0))))
	      (progn 
		(setq compared-str (substring s0 0 (length s1)))
		(setq target-str s1)))
	    (if (string-equal compared-str target-str)

;;	    (if (string-equal s0 s1)
		(progn 
		  (goto-char (manued-get-doc-begin-point))
		  (if (search-forward (car compared-def) nil t)
		      (setq hilit-err-occur-pos (match-beginning 0)))
		  (error (format 
			  "%s and %s are the same, check your defcommand."
			  (car compared-def) (car target-def))))))
	  (setq target-list (cdr target-list))))
      (setq compared-list (cdr compared-list)))))

;;;------------------------------------------------------------
;;; manued-order-str $B$NCM$K$h$C$F(B order $B$r7h$a$k(B
;;;------------------------------------------------------------
;; order indicating string 
(defvar manued-order-str "older-first"
  "* delete order. 
The form of the manued command is [first/last]. ``older''
indicates original document and ``newer'' indicates revised
document. This string sets the variable `manued-is-order-older-first'. 

$B??s,F;>C5n%3%^%s%I$NE,MQ=g!%>C5n%3%^%s%I$NFbMF$r(B [first/last] 
$B$H$9$k!%(B `older' $B$O85J8=q$r!$(B`newer' $B$OJQ998e$NJ8=q$r<($9!%$3$NJ8;zNs(B
$B$K$7$?$,$C$F(B `manued-is-order-older-first' $B$NCM$,%;%C%H$5$l$k!%(B

-------------------------+-----------------+----------------------------
manued-order-str         | change from to  | manued-is-order-older-first
-------------------------+-----------------+----------------------------
 older-first, newer-last |  first -> last  |           t
 older-last,  newer-first|  last  -> first |          nil
-------------------------+-----------------+----------------------------")

(defvar manued-is-order-older-first t
  "Applying swap-command order. 
When t, [first/last] will change first -> last.

$B>C5n%3%^%s%I$N(B [first/last] $B$N$I$A$i$,D{@5@h$+$r<($9!%(B
[$BD{@5A0(B/$BD{@58e(B]$B$N>l9g$K(B t$B!$$=$N5U(B ([$BD{@58e(B/$BD{@5A0(B]) $B$N;~$K(B nil$B!%(B") 

;;
;; set delete command order according to order string 
;;	order $BJ8;zNs$K=>$C$F(B delete command $B$NA08e$r7hDj$9$k(B
;;
(defun manued-set-order-from-order-str ()
  "set delete command order according to order string."
  (cond ((member manued-order-str '("older-first" "newer-last"))
	 (setq manued-is-order-older-first t))
	((member manued-order-str '("older-last" "newer-first"))
	 (setq manued-is-order-older-first nil))
	(t 
	 (setq manued-order-str "older-first")
	 (setq manued-is-order-older-first t)
	 (goto-char (manued-get-doc-begin-point))
	 (search-forward "deforder")
	 (setq hilit-err-occur-pos (match-beginning 0))
	 (error (format "illeal deforder str (setted %s)" manued-order-str)))))

;;------------------------------------------------------------
;; insert manued header
;;------------------------------------------------------------
(defvar manued-is-auto-insert-header '(t t nil)
  "setting for defcommands insertion 

'(is-auto-insert is-query-when-insert insert-point)

When `is-auto-insert' is t, manued.el inserts defcommands in the
current buffer in the case of no defcommands in the buffer.  If
`is-auto-insert' is nil, manued.el does not insert defcommands.

When the second element `is-query-when-insert' is t, manued.el asks to
user `insert-manued header (y or n)', otherwise no question. This is
only effective when `is-auto-insert' is t.

The third element of this list `insert-point' indicates insert point
of defcommands. The meaning of value is as folowing.

    t         current point
    nil       (point-min)
    number    point as a number

$B%P%C%U%!$K(B defcommand $B$rA^F~$9$k>l9g$N@_Dj(B

`is-auto-insert' $B$,(B t $B$N>l9g!$$b$78=:_$N%P%C%U%!$K(Bdefcommand$B$,L5$$>l9g$K(B
$B$O(Bmanued.el$B$O(Bdefcommand$B$r8=:_$N%P%C%U%!$KA^F~$7$^$9!%(Bnil$B$N>l9g$K$O2?$b$7(B
$B$^$;$s!%(B

2$BHVL\$NMWAG(B`is-query-when-insert'$B$,(Bt$B$N>l9g!$(Bmanued.el$B$O%f!<%6$K(B
defcommand$B$rA^F~$7$FNI$$$+?R$M$k$h$&$K$J$j$^$9!%(Bnil$B$N>l9g$K$O?R$M$^$;$s!%(B
$B$3$l$O!$(B`is-auto-insert'$B$,(Bt$B$N>l9g$KM-8z$G$9!%(B

3$BHVL\$NMWAG$G$"$k(B`insert-point'$B$O(Bdefcommand$B$NA^F~0LCV$r;XDj$9$k$b$N$G!$(B
$B<!$N$h$&$J0UL#$r;}$A$^$9!%(B

    t         $B8=:_$N%]%$%s%H(B
    nil       (point-min)
    number    $B?t;z$G<($7$?(Bpoint$B0LCV(B
")

(defun manued-insert-header (p)
  "insert manued header at point"
  (interactive "d")
  (goto-char p)
  (insert (format "defparentheses\t%s %s\n" 
		  manued-l-parenthesis-str manued-r-parenthesis-str))
  (let ((defalist manued-def-alist) (item))
    (while defalist
      (setq item (car defalist))
      (or (member (car item) '("defLparenthesis" "defRparenthesis"))
	  (insert 
	   ;; (format "%s\t%s\n" (car item) (eval (car (cdr item)))))
	   ;; $B$H(B eval $B$r;H$C$F$$$?$,!$(Bsymbol-value $B$r;H$&(B
	   (format "%s\t%s\n" (car item) (symbol-value (car (cdr item))))))
      (setq defalist (cdr defalist)))))
;;
;; when non exist header,  insert manued header with quary
;;
(defun manued-search-and-insert-header ()
  "$B??s,F;(B def $B%3%^%s%I$r%5!<%A$7!$B8:_$7$J$$>l9g$K$OA^F~$9$k$+?R$M$k!%(B

see variable : manued-is-auto-insert-header"
  (save-excursion 
    (if (and (car manued-is-auto-insert-header)
	     (not manued-header-is-found)) ; mode $B$KF~$k:]$KI,$:8F$P$l$F$$$k(B
	(let ((p) (iposinfo (car (cdr (cdr manued-is-auto-insert-header)))))
	  (cond ((eq t iposinfo) 
		 (setq p (point)))
		((null iposinfo) 
		 (setq p (point-min)))
		((and (numberp iposinfo) (> iposinfo 0))
		 (setq p iposinfo)))
	  (if (car (cdr manued-is-auto-insert-header))
	      (if (y-or-n-p "insert manued header?")
		  (manued-insert-header p))
	    (manued-insert-header p))))))

;;============================================================
;; hilit manued commands
;;============================================================
;; recenter hilit
;;------------------------------
(defun manued-recenter-hilit ()
  "recenter and hilit 

When color mode is enable, recenter and hilight. But when color mode
is disabled, only recenter.

recenter $B$7$?8e$K??s,F;%3%^%s%I$r(B hilit $B$9$k!%$?$@$7!$(Bhilit $B$r9T$&$N$O(B 
color mode $B$,(B off $B$N;~$N$_!%(B"
  (interactive)
  (manued-hilit)
  (recenter))

;;------------------------------
;; $B8=:_(B hilit $B$9$Y$-$b$N$r<($9(B.
;; manued-hilit $B$,8F$P$l$?;~$K$O$3$NJQ?t$r8+$F(B hilit-older $B$+(B 
;; hilit-newer $B$+$rH=Dj$9$k!%(Bt $B$N$H$-$K$O(B newer $B$r(B hilit $B$9$k!%(B
;;------------------------------
(defvar manued-is-now-hilit-newer t)

;;------------------------------
;; hilit
;;------------------------------
(defun manued-hilit ()
  "hilit manued command.
When `manued-use-color-hilit' is t, hilit manued command according to
the value of `manued-is-now-hilit-newer'.

$B??s,F;%3%^%s%I$r(B hilit $B$9$k!%(B
$B$b$7(B `manued-use-color-hilit' $B$,(B t $B$J$i$P(B manued command $B$r(B hilit $B$9$k!%(B
$B$3$N;~!$(B`manued-is-now-hilit-newer' $B$,(B t $B$J$i$P(B newer $B$r(B hilit $B$7!$(Bnil 
$B$J$i$P(B older $B$r(B hilit $B$9$k!%(B"
  (if (manued-guess-color-mode)
      (progn 
	(if manued-is-now-hilit-newer
	    (manued-hilit-newer)
	  (manued-hilit-older)))
    (manued-unhilit-current-buffer)))

;;------------------------------
;; unhilit current buffer
;;------------------------------
(defun manued-unhilit-current-buffer ()
  (remove-text-properties (point-min) (point-max) '(face nil)))

;;
;; hilit newer
;;
(defun manued-hilit-newer ()
  "hilit newer part within manued command.
$BJ8>OCf$N??s,F;%3%^%s%ICf$ND{@58eItJ,$r%O%$%i%$%H$9$k!%(B"
  (interactive)
  (let ((hilit-err-occur-pos nil))
    (condition-case err-message
	(save-excursion
	  (manued-init-vars)
	  (goto-char (manued-get-doc-begin-point))
	  (setq manued-is-now-hilit-newer t)
	  (if manued-is-order-older-first
	      (manued-set-hilit-color manued-last-hilit-color-list)
	    (manued-set-hilit-color manued-first-hilit-color-list))
	  (while (manued-hilit-manuedexp)))
      (error       ; error handling
       (if hilit-err-occur-pos
	   (goto-char hilit-err-occur-pos))
       (error (format "Error! : manued-hilit-newer : %s" err-message))))))

;;
;; hilit older
;;
(defun manued-hilit-older ()
  "hilit older part within manued command.
$BJ8>OCf$N??s,F;%3%^%s%ICf$ND{@5A0ItJ,$r%O%$%i%$%H$9$k!%(B"
  (interactive)
  (let ((hilit-err-occur-pos nil))
    (condition-case err-message
	(save-excursion
	  (manued-init-vars)
	  (goto-char (manued-get-doc-begin-point))
	  (setq manued-is-now-hilit-newer nil)
	  (if manued-is-order-older-first
	      (manued-set-hilit-color manued-first-hilit-color-list)
	    (manued-set-hilit-color manued-last-hilit-color-list))
	  (while (manued-hilit-manuedexp)))
      (error       ; error handling
       (if hilit-err-occur-pos
	   (goto-char hilit-err-occur-pos))
       (error (format "Error! : manued-hilit-older : %s" err-message))))))
     
;;
;; hilit toplevel (hilit-manuedexp see manued.grammer)
;;	lap ... LookAhead Pstr
;;
(defun manued-hilit-manuedexp ()
  "hilit all manued command at top level. 
$B??s,F;$NJ8=qCf$N%3%^%s%I$rA4$F%O%$%i%$%H$9$k!%(B"
  (let ((cont t))
    (while cont 
      (let ((lap (manued-search-nonescaped-command-in-hirabun ; [ $B$rC5$9(B
		  (manued-get-doc-end-point))))
	(if lap				; found [. $B$3$N(B lap $B$O(B [ $B$G$"$k(B
	    (progn
	      (manued-hilit-one-command lap) 
	      ;; colored-pos-pstr $B$O$3$l$^$G?'$rEI$C$?>l=j$r<($9Bg0hE*(B
	      ;; $B%]%$%s%?!$:G=i$N(B region $B$O(B delete-first $B$H2>Dj$9$k(B
	      (let ((colored-pos-pstr lap)) 
		(manued-hilit-manued-term 'manued-com-delete-first)))
	  (setq cont nil))))))		; not found

;;
;; hilit manued-term : LL(1), see Dragon book.
;;
(defun manued-hilit-manued-term (cur-command)
  (let ((cont t) 
	(lap nil))			; lookahead-pstr 
    (while cont
      (setq lap (manued-search-nonescaped-command 
		 (manued-all-command-pat) (manued-get-doc-end-point)))
      (if (null lap)
	  ;; No command is found. At least, `]' must be found here.
	  ;; This must be error.
	  (progn			
	    (manued-hilit-can-not-find-end-paren colored-pos-pstr)
	    (setq cont nil))
	(progn
	  (manued-hilit-one-command lap) ; $B%3%^%s%I=hM}(B
	  ;; $B$3$l$^$G$NHO0O$r=hM}(B
	  (cond 
	   ;; ; comment in and exit this level
	   ((manued-command-eq lap manued-comment-str)
	    (if (or (eq cur-command 'manued-com-delete-first) 
		    (eq cur-command 'manued-com-delete-last)
		    (eq cur-command 'manued-com-swap-gamma))
		(progn (manued-hilit-one-region 
			colored-pos-pstr lap cur-command)
		       ;; $B<!$N(B cur-command $B$O(B hilit-commet $B$,%3%a%s%H(B
		       ;; $B$H$$$&$3$H$rCN$C$F$$$k$N$G>iD9$@$,!$%W%m%0%i(B
		       ;; $B%`$H$7$F$N0l4S@-$r<h$k$?$a$K$"$($F(B setq $B$9$k(B
		       (setq cur-command 'manued-com-comment)
		       (manued-hilit-comment lap)
		       (setq cont nil))
	      (progn 
		(setq hilit-err-occur-pos (manued-get-first-point lap))
		(error "illegal command, is command right?"))))
	   
	   ;; [ recursion
	   ((manued-command-eq lap manued-l-parenthesis-str) 
	    (manued-hilit-one-region colored-pos-pstr lap cur-command)
	    (manued-hilit-manued-term 'manued-com-delete-first))
	   
	   ;; / delete 
	   ((manued-command-eq lap manued-delete-str) 
	    (manued-hilit-one-region colored-pos-pstr lap cur-command)
	    (setq cur-command 'manued-com-delete-last))
	   
	   ;; | swap
	   ((manued-command-eq lap manued-swap-str) 
	    (cond 
	     ((eq cur-command 'manued-com-delete-first)
	      (setq cur-command 'manued-com-swap-alpha) ; $B$3$3$G(Bswap$B$H$o$+$k(B
	      (manued-hilit-one-region colored-pos-pstr lap cur-command)
	      (setq cur-command 'manued-com-swap-beta))
	     ((eq cur-command 'manued-com-swap-beta)
	      (manued-hilit-one-region colored-pos-pstr lap cur-command)
	      (setq cur-command 'manued-com-swap-gamma))
	     (t
	      (setq hilit-err-occur-pos (manued-get-first-point lap))
	      (error "illegal command, here must be swap command region."))))
	   
	   ;; ] 
	   ((manued-command-eq lap manued-r-parenthesis-str) 
	    (manued-hilit-one-region colored-pos-pstr lap cur-command)
	    (setq cont nil)) ; return; exit loop
	   
	   (t
	    (setq hilit-err-occur-pos (manued-get-first-point lap))	    
	    (error "Internal error. I do not know such command."))))))))

;;
;; $B=*N;$N3g8L$,$_$D$+$i$J$$>l9g$N%(%i!<=hM}(B
;;
(defun manued-hilit-can-not-find-end-paren (pstr)
  (setq hilit-err-occur-pos (manued-get-first-point pstr))
  (error (format "hilit-one-level : lack of `%s' of this `%s'"
		 manued-r-parenthesis-str manued-l-parenthesis-str)))
;;
;; hilit-commet 
;;
(defun manued-hilit-comment (pstr)	    
  (let ((lap nil))
    (setq lap (manued-search-nonescaped-command 
	       (manued-outof-command-pat) (manued-get-doc-end-point)))
    (if (or (null lap) (not (manued-command-eq lap manued-r-parenthesis-str)))
	(progn 
	  (setq hilit-err-occur-pos (manued-get-first-point pstr))	  
	  (error "Missing r-parenthesis for the end of comment."))
      (progn 
	(manued-hilit-one-command lap)
	(manued-hilit-one-region colored-pos-pstr lap 'manued-com-comment)))))

;;
;; $B0l$D$N(B manued $B%3%^%s%I(B (ex. [, ], /, |, ;) $B$N(B hilit
;;
(defun manued-hilit-one-command (pstr)
  "$B0l$D$N??s,F;%3%^%s%I$r(B hilit $B$9$k!%(B"
  (put-text-property (manued-get-first-point pstr)
		     (manued-get-end-point   pstr)
		     'face 
		     'manued-command-face))

;;
;; $B;XDj$7$??'$G;XDjHO0O$r(B hilit 
;;
(defun manued-hilit-one-color (begin-pstr end-pstr color)
  "$B??s,F;%3%^%s%ICf$NO"B3$7$?ItJ,$r;XDj$7$??'$G(B hilit $B$9$k!%(B

hilit a manued region with indicated color-face."
  (put-text-property  (manued-get-end-point   begin-pstr)
		      (manued-get-first-point end-pstr)
                      'face
		      color))

;;
;; $B$"$kHO0O$r?'$rA*Br$7$F%O%$%i%$%H$9$k(B
;; $B$I$3$^$G?'$rEI$C$?$+$r(B colored-pos-pstr $B$K5-O?$9$k(B
;;  
(defun manued-hilit-one-region (beg-pstr end-pstr cur-command)
  "hilit a manued command region. 
$B0l$D$N??s,F;%3%^%s%I$NHO0O$r;XDj$9$k$H$=$NHO0O$r%O%$%i%$%H$9$k!%(B"
  (cond ((eq cur-command 'manued-com-delete-first) ; [first/]
	 (manued-hilit-one-color beg-pstr end-pstr 'manued-delete-first-face))
	((eq cur-command 'manued-com-delete-last) ; [/last]
	 (manued-hilit-one-color beg-pstr end-pstr 'manued-delete-last-face))
	((eq cur-command 'manued-com-swap-alpha)  ; [alpha||]
	 (manued-hilit-one-color beg-pstr end-pstr 'manued-swap-alpha-face))
	((eq cur-command 'manued-com-swap-beta)   ; [|beta|]
	 (manued-hilit-one-color beg-pstr end-pstr 'manued-swap-beta-face))
	((eq cur-command 'manued-com-swap-gamma)  ; [||gamma]
	 (manued-hilit-one-color beg-pstr end-pstr 'manued-swap-gamma-face))
	((eq cur-command 'manued-com-comment)     ; [;comment]
	 (manued-hilit-one-color beg-pstr end-pstr 'manued-comment-face))
	(t (error "Unknown manued command and color.")))
  (setq colored-pos-pstr end-pstr))

;;
;; set hilit color
;;
(defun manued-set-hilit-color (color-val-list)
  "hilit $B?'$N@_Dj(B"
  (let ((color-sym '(manued-delete-first-face
		     manued-delete-last-face
		     manued-swap-alpha-face
		     manued-swap-beta-face
		     manued-swap-gamma-face
		     manued-comment-face
		     manued-command-face))
	(color-val color-val-list))
    ;; (mapcar 'set color-sym color-val) $B$O(B elisp $B$G$O$G$-$J$$!%0z?t$O0l$D(B
    (while color-sym
      (let ((face-sym (car color-sym)))
	(make-face face-sym)
	(set-face-foreground face-sym (car color-val))
	(setq color-sym (cdr color-sym))
	(setq color-val (cdr color-val))))))

;;============================================================
;; search manued command
;;============================================================
;;
;; search-nonescaped-command (coms-regpat end-point) 
;; $B%(%9%1!<%W$r9M$($F%3%^%s%I$r%5!<%A$9$k(B
;;
;; $B%(%9%1!<%W$N0UL#$O!$!V%(%9%1!<%WJ8;z$N<!$NJ8;z$OHt$P$9!W$H$$$&$3$H(B
;; $B$KDj5A$9$k(B
;;
(defun manued-search-nonescaped-command (coms-regpat end-point)
  "$B%(%9%1!<%W$r9MN8$7$F%3%^%s%I$r%5!<%A$9$k!%(B
search-coms-regpat : $B%5!<%A$9$k(B command $BJ8;z$N(B regex $B%Q%?!<%s(B
end-point          : $B$I$3$^$GC5$9$+(B(point)

escape $B$5$l$F$$$J$$%3%^%s%I$rC5$7!$$_$D$+$C$?$i$=$N%3%^%s%I$N(B
``begin-point end-point $B%3%^%s%IJ8;zNs(B'' $B$N%j%9%H$rJV$9!%$_$D$+$i$J(B
$B$+$C$?$i(B nil$B$rJV$9(B"
  (catch 'tag
    (while t	
      (let ((find-com (manued-search-command coms-regpat end-point)))
	(if (not find-com)
	    (throw 'tag nil)		; $B%3%^%s%I$O$J$$(B
	  ;; escape $BJ8;z$N>l9g$K$O%5!<%A$9$k(B escape $B$H(B L-parenthesis 
	  ;; $B$H$NAH$GM?$($i$l$F$$$k!%(Bex. ~[ 
	  (let (; (fpos (manued-get-first-point find-com))  ; $B%3%^%s%I72$N:G=i(B
		(epos (manued-get-end-point   find-com))) ; $B=*$o$j(B
	    (goto-char (manued-get-first-point find-com)) ; $B%3%^%s%I0LCV$X0\F0(B
	    (if (looking-at (regexp-quote manued-escape-str)) ; escape $BJ8;z$+(B?
		(progn			; escape str $B$@$C$?(B 
		  (goto-char epos)	; esc$B$N8e$X0\F0(B ESCPAT^COMPAT
		  (forward-char 1))	; 1$BJ8;zHt$P$9(B
	      (progn			; escape $BJ8;z$G$J$+$C$?(B
		(goto-char epos)	; 'COMPAT^'
		(throw 'tag find-com)))))))))

;;
;; serach-command (search-coms-regpat end-point)
;; ex.	(manued-search-command "$B!Z(B\\|$B![(B\\|/\\|;\\|~" 
;;	  (manued-get-doc-end-point))
;;	$B%3%^%s%I$N$_$r%5!<%A$9$k(B
;;
(defun manued-search-command (search-coms-regpat end-point) 
  "manued $B%3%^%s%I$rC5$9(B. $B%(%9%1!<%W$O9MN8$7$J$$!%(B
search-coms-regpat : $B%5!<%A$9$k(B command $BJ8;z$N(B regex $B%Q%?!<%s(B
end-point          : $B$I$3$^$GC5$9$+(B(point)

$B%^%C%A$7$?%3%^%s%I$N(B (begin-point end-point ``$B%3%^%s%IJ8;zNs(B'') $B$N(B
$B%j%9%H$rJV$9!%$_$D$+$i$J$+$C$?$i(B nil $B$rJV$9!%HO0O$N30$+$iC5$7;O$a$k>l(B
$B9g$K$O(B re-search-forward $B$N(B NOERRROR $B$O8z$+$J$$$i$7$$$N$GBP:v$r9V$8$F(B
$B$*$/!%(B" 
  (if (<= end-point (point))
      nil
    (if (re-search-forward search-coms-regpat end-point t)
	(list (match-beginning 0)
	      (match-end 0)
	      (buffer-substring (match-beginning 0) (match-end 0)))
      nil)))

;;
;; search-command-in-hirabun
;;  $BJ?J8Cf$G??s,F;%3%^%s%I$rC5$9(B
;;
;;  $BJ?J8Cf$O(B `~' $B$,%3%^%s%I$G$J$$$K$b$+$+$o$i$:!$(B`~[' $B$O%(%9%1!<%W$5$l(B
;;  $B$J$/$F$O$J$i$J$$$H$$$&FC<l$JJ}<0$K$7$?!%$3$l$O;H$$0W$5$N$?$a$G$"$k!%(B
;;  $BJ?J8Cf$G9MN8$7$J$/$F$O$J$i$J$$J8;z$r(B `~' $B$H(B `[' $B$K$9$k$N$G$O$J$/!$(B
;;  `[' $B$@$1$K$7$?$$$H9M$($?!%(B
;;
(defun manued-search-nonescaped-command-in-hirabun (end-point)
  "search manued command in normal text region

end-point : where to search.

$B%(%9%1!<%W$5$l$F$$$J$$J?J8Cf$N??s,F;%3%^%s%I$N;O$^$j$rC5$9(B"
  (catch 'tag
    (while t	
      (let ((find-com (manued-search-command 
		       (manued-hirabun-command-pat) end-point)))
	(if (not find-com)
	    (throw 'tag nil)		; $B%3%^%s%I$O$J$$(B
	  (if (not (manued-command-eq find-com (manued-escaped-l-paren-pat)))
	      (throw 'tag find-com)))))))

;;------------------------------------------------------------
;; $B%3%^%s%IJ8;z=89g(B
;;------------------------------------------------------------
;; $BJ?J8$G$O(B [ $B$+(B ~[ $B$r=hM}$9$k(B
;;
(defun manued-hirabun-command-pat ()
  "$BJ?J8$+$i??s,F;$KF~$k>l9g$N%3%^%s%IJ8;z=89g$rF@$k(B"
  (concat
   (regexp-quote manued-l-parenthesis-str) "\\|" 
   (manued-escaped-l-paren-pat)))

;; ] 
(defun manued-outof-command-pat ()
  "$B??s,F;%3%^%s%I$+$i=P$k>l9g$N%3%^%s%IJ8;z=89g$rF@$k(B"
  (concat 
   (regexp-quote manued-r-parenthesis-str) 	"\\|" 
   (regexp-quote manued-escape-str)))


;; $BA4%3%^%s%IJ8;z(B
(defun manued-all-command-pat ()
  "$B??s,F;A4%3%^%s%IJ8;z=89g$rF@$k(B"
  (concat 
   (regexp-quote manued-l-parenthesis-str) 	"\\|" 
   (regexp-quote manued-r-parenthesis-str) 	"\\|" 
   (regexp-quote manued-delete-str)		"\\|" 
   (regexp-quote manued-swap-str)		"\\|" 
   (regexp-quote manued-comment-str)		"\\|" 
   (regexp-quote manued-escape-str)))

;; ~[ $BJ8;z(B
(defun manued-escaped-l-paren-pat ()
  (concat (regexp-quote manued-escape-str)
	  (regexp-quote manued-l-parenthesis-str)))

;; regrex $B$K;H$($J$$$,H/8+$7$?J8;z$N%^%C%A$K;H$&(B ~[ $BJ8;z(B
(defun manued-escaped-l-paren-str ()
  (concat manued-escape-str manued-l-parenthesis-str))

(defvar manued-ask-if-formatted-buffer-is t
  "* $B4{$K@07A:Q$_$N(B buffer $B$,B8:_$7$?>l9g$K?R$M$k$+$I$&$+!%(Bt $B$G?R$M$F$/$k(B")

;; 1998$BG/(B6$B7n(B27$BF|(B($BEZ(B)
;; $B@hF|!$CN?M$,;`$s$@!%K\F|Ar<0$K=P$?!%$d$j$-$l$L!%$^$C$?$/!$$d$j$-$l$s!%(B

;;;============================================================
;;; $B?7$?$J(B manued $B@07AMQ%P%C%U%!$N:n@.(B 
;;;============================================================
;; $B?7$?$J%P%C%U%!$N:n@.(B
(defun manued-get-format-buffer ()
  "get a formatting buffer for manued.

If no manued buffer, create and return it. Otherwise, ask to the user
overwrite or not. However, if manued-ask-if-formatted-buffer-is is
nil, never ask and override the buffer.

$B??s,F;$N@07AMQ$N%P%C%U%!$r:n@.$9$k!%L5$$>l9g$K$O:n$C$FJV$9!%M-$k>l9g$K(B
$B$O>C$7$FNI$$$+?R$M$k!%$?$@$7!$(Bmanued-ask-if-formatted-buffer-is $B$,(B nil 
$B$N;~$K$O?R$M$:$K:n@.$9$k!%(B"
  (if (null (get-buffer manued-formatted-buffer-name))
      (get-buffer-create manued-formatted-buffer-name)
	;; $B4{$KB8:_$9$k(B
    (if manued-ask-if-formatted-buffer-is
	(let ((prompt (format "buffer %s is already exist. clear? " 
			      manued-formatted-buffer-name)))
	  (if (y-or-n-p prompt)
	      (let ((curbuf (current-buffer)))
		(switch-to-buffer manued-formatted-buffer-name)
		(kill-buffer manued-formatted-buffer-name)
		(switch-to-buffer curbuf)		
		(get-buffer-create manued-formatted-buffer-name))
	    nil)))))

;;============================================================
;; pretty print control 
;;============================================================
(defun manued-set-pretty-print-off ()
  "set manued pretty print off"
  (interactive)
  (setq manued-pretty-print-on-p nil))

(defun manued-set-pretty-print-on ()
  "set manued pretty print on"
  (interactive)
  (setq manued-pretty-print-on-p t))

;;============================================================
;; show : $B@07A%3%^%s%I(B
;;============================================================
;;
;; show newer
;;
(defvar manued-show-newer-p nil)
(defun manued-show-newer-in-manued-buffer ()
  "Show revised document from current manued document to another buffer.
$B8=:_$N%P%C%U%!$N??s,F;J8=q$NJQ998e$NJ8=q$rB>$N%P%C%U%!$KI=<($9$k!%(B"
  (interactive)
  (manued-show-in-manued-buffer t manued-pretty-print-on-p))

;;
;; show newer document with region
;;
(defun manued-show-newer-region (b e)
  "Show newer version with region.
$B%j!<%8%g%s$NHO0O$r@07A$7$F?7$7$$J8=q$r<h$j=P$9(B"
  (interactive "r")
  (setq manued-show-newer-p t)
  ;; (dbg (format "b = %d, e = %d" b e))
  (manued-show-region b e))

;;
;; show older
;;
(defun manued-show-older-in-manued-buffer ()
  "Show original document from current manued document to another buffer.
$BJ8>OCf$N??s,F;%3%^%s%I$NJQ99A0$NJ8=q$rB>$N%P%C%U%!$KI=<($9$k!%(B"
  (interactive)
  (manued-show-in-manued-buffer nil manued-pretty-print-on-p))

;;
;; show older document with region
;;
(defun manued-show-older-region (b e)
  "Show older version with region.
$B%j!<%8%g%s$NHO0O$r@07A$7$F8E$$J8=q$r<h$j=P$9(B"
  (interactive "r")
  (setq manued-show-newer-p nil)
  (manued-show-region b e))

;;
;; show revised document at another buffer
;;	$BB>$N%P%C%U%!$r:n$j!$$=$3$KFbMF$r%3%T!<$7$F@07A$9$k(B
;;
(defun manued-show-in-manued-buffer (show-newer-p pretty-print-p)
  "Show processed manued document to another buffer. 

When show-newer-p is t, newer document is shown. Otherwise, older
document is shown.

$BJ8>OCf$N??s,F;%3%^%s%ICf$N85J8=q$rB>$N%P%C%U%!$KI=<($9$k!%(B
show-newer-p $B$,(B t $B$J$i?7$7$$J}$rI=<($7!$(Bnil $B$J$i8E$$J}$rI=<($9$k!%(B"
  (let ((formatbuf (manued-get-format-buffer)))
    (if formatbuf
	(let ((orgbuf (current-buffer)))	; $B8=:_$N(B buffer
	  (pop-to-buffer formatbuf)
	  (insert-buffer orgbuf)
	  (setq manued-show-newer-p      show-newer-p)
	  (setq manued-pretty-print-on-p pretty-print-p)
	  (manued-show-buffer)))))

;;
;; $B8=:_$N%P%C%U%!$N??s,F;%3%^%s%I$N@07A$r9T$&(B
;;
(defun manued-show-buffer ()
  "$B8=:_$N%P%C%U%!$N??s,F;%3%^%s%I$r@07A$9$k!%(B
$B8=:_$N(B manued-show-newer-p $B$,<($9ItJ,$r<h$j=P$9!%(B"
  (let ((replace-err-occur-pos nil))
    (condition-case err-message
	(save-excursion
	  (manued-init-vars)
	  (manued-show-region (manued-get-doc-begin-point)
			      (manued-get-doc-end-point)))
      (error				; error handling
       (if replace-err-occur-pos
	   (goto-char replace-err-occur-pos))
       (error (format "Error! : %s" err-message))))))

;;
;; $B8=:_$N%P%C%U%!$N(B region $B$G;XDj$5$l$?HO0O$N??s,F;%3%^%s%I$N@07A$r9T$&(B
;;
(defun manued-show-region (b e)
  "$B8=:_$N%P%C%U%!$N??s,F;%3%^%s%I$r@07A$9$k!%(B
$B8=:_$N(B manued-show-newer-p $B$,<($9ItJ,$r<h$j=P$9!%(B"
  (goto-char e)
  (let ((region-end-marker (point-marker))) ; marker of the end of the region 
    (goto-char b)    
    (while (manued-replace-manuedexp region-end-marker))
    (goto-char (marker-position region-end-marker))
    (set-marker region-end-marker nil)))

;;
;; replace-toplevel
;;
(defun manued-replace-manuedexp (region-end-marker)
  "$B??s,F;J8=q$NJ?J8Cf$N%3%^%s%I$r=hM}$9$k!%(B" 
  (let ((cont t))
    (while cont
      (let ((lap 
	     (manued-search-command (manued-hirabun-command-pat)
				    (marker-position region-end-marker))))
	(cond 
	 ((eq lap nil)			; lap is nil ... command is not found.
	  (setq cont nil))		; $BA4$F=hM}$7$?(B: exit loop

	 ;; ~[ $BJ?J8Cf$N%(%9%1!<%W3g8L$N=hM}(B
	 ((string-equal (manued-escaped-l-paren-str)
			(manued-get-command-str lap))
	  (let ((m (make-marker)))
	    (set-marker m (manued-get-end-point lap)) ; ~[^ $B$N=*$o$j$K%^!<%/(B
	    (manued-proc-escape (manued-get-first-point lap) 
				(manued-get-end-point   lap))
	    (goto-char (marker-position m))
	    (set-marker m nil)))

	 ;; [ $B??s,F;%3%^%s%I$N=hM}(B
	 (t
	  (manued-replace-manued-term lap region-end-marker)))))))

;;
;; $B$^$::GFb$N(B [] $B$r$_$D$1$k$=$7$F$=$N:GFb??s,F;%3%^%s%I$+$i:F5"E*$K=hM}(B
;; $B:F5"E*$K=hM}$r$7$F$bJ8=q$NA0$NJ}$G8!=P$7$?(B point $B$OJQ2=$7$J$$(B
;;
;; lap = LookAhead-Pstr
;;
(defun manued-replace-manued-term (beg-lap region-end-marker)
  (let ((cont t) 
	(lap nil)			; lookahead-pstr 
	(swap-pstr-list   '())		; swap   $B5-9f$N%j%9%H(B
	(delete-pstr-list '())		; delete $B5-9f$N%j%9%H(B
	)			
    (while cont
      (setq lap (manued-search-nonescaped-command 
		 (manued-all-command-pat) (marker-position region-end-marker)))
      (if (null lap)
	  ;; Error. No command is found. At least, `]' must be found here.
	  (progn 
	    (manued-replace-can-not-find-end-paren beg-lap)	    
	    (setq cont nil))
	(cond 
	 ;; ; comment 
	 ((manued-command-eq lap manued-comment-str)
	  (let ((comment-pstr lap))
	    (setq lap (manued-search-nonescaped-command 
		       (manued-outof-command-pat) 
		       (marker-position region-end-marker)))
	    (if (not (null lap))
		(progn
		  (manued-replace-one-term swap-pstr-list delete-pstr-list
					   beg-lap lap comment-pstr)
		  (setq cont nil))
	      (progn 
		(manued-replace-can-not-find-end-paren beg-lap)	    
		(setq cont nil)))))
	 ;; [, recursion
	 ((manued-command-eq lap manued-l-parenthesis-str) 
	  (manued-replace-manued-term lap region-end-marker))
	 ;; / delete 
	 ((manued-command-eq lap manued-delete-str) 
	  (setq delete-pstr-list (cons lap delete-pstr-list)))
	 ;; | swap
	 ((manued-command-eq lap manued-swap-str) 
	  (setq swap-pstr-list (cons lap swap-pstr-list)))
	 ;; ] 
	 ((manued-command-eq lap manued-r-parenthesis-str) 
	  (manued-replace-one-term swap-pstr-list delete-pstr-list 
				   beg-lap lap nil)
	  (setq cont nil))		; return; exit loop
	 ;; error
	 (t
	  (error "Internal error. I do not know such replace command.")))))))

;;
;; $B%3%^%s%I$N@09g@-$N%A%'%C%/(B
;;   beg-lap $B$O%(%i!<$NH/@8>l=j$r<($9$?$a(B
;;
(defun manued-check-command (swap-symcount delete-symcount beg-lap)
  (catch 'tag
    (let ((permitted-occur-num-list	; $B5v$5$l$kAH9g$;(B
	   '((0 0)			; []
	     (0 1)			; [/]
	     (2 0))))			; [||]
      (while (not (null permitted-occur-num-list))
	(let ((comb (car permitted-occur-num-list)))
	  (if (and (= swap-symcount (car comb)) 
		   (= delete-symcount (car (cdr comb))))
	      (throw 'tag t))
	  (setq permitted-occur-num-list (cdr permitted-occur-num-list))))
      (goto-char (manued-get-first-point beg-lap))
      (if (> swap-symcount 0)
	  (error (format "Illegal swap command. %s can appear only 0/2 times."
			 manued-swap-str))
	(error (format "Illegal delete command. %s can appear only 0/1 times."
		       manued-delete-str))))))


;;
;; $B%3%^%s%I$K$h$C$F@07A%3%^%s%I$r8F$S=P$9(B
;;
(defun manued-replace-one-term (swap-pstr-list delete-pstr-list 
					       beg-lap end-lap comment-psr) 
  (let ((swap-symcount   (length swap-pstr-list))
	(delete-symcount (length delete-pstr-list)))
    ;; consistency check
    (manued-check-command swap-symcount delete-symcount beg-lap)
    ;; replace
    (cond
     ((eq swap-symcount 2)
      (manued-replace-one-swap-term 
       swap-pstr-list beg-lap end-lap comment-psr)
      )
     ((eq delete-symcount 1)
      (manued-replace-one-delete-term 
       delete-pstr-list beg-lap end-lap comment-psr)
      )      
     (t
      (manued-replace-one-null-term beg-lap end-lap comment-psr)))))

;;
;; get pretty print strings
;; 
(defun manued-get-pretty-print-str (format-str arg-str)
  ;; if there is args, output anyway
  (let ((res ""))
    (if (not (null arg-str))
	(setq res (format format-str arg-str))
      (if manued-pretty-print-null-comment-out-p
	  (setq res (format format-str "")))
      )
    res))

;;
;; swap $B%3%^%s%I$r=hM}$9$k(B
;; 	[alpha|beta|gamma;comment]
;; $B%"%k%4%j%:%`$N35MW(B
;; $B>C$7$F(B insert $B$9$k$3$H$GF~$l49$($r9T$&!%(Binsert $B$N8e$K%]%$%s%H$,Mh$k(B
;; $B$N$G$=$N>l$G%^!<%/$9$k$HHO0O$N:G8e$,$o$+$k!%$=$3$G(B escape $B$r=hM}$7!$(B
;; $B:G8e$K0\F0$9$k!%(B
;;
(defun manued-replace-one-swap-term (swap-pstr-list
				     beg-lap end-lap comment-psr)
  (let ((end-marker  (make-marker))
	(beg-alpha   (manued-get-end-point    beg-lap))
	(end-alpha   (manued-get-first-point (car (cdr swap-pstr-list))))
	(beg-beta    (manued-get-end-point   (car (cdr swap-pstr-list))))
	(end-beta    (manued-get-first-point (car  swap-pstr-list)))
	(beg-gamma   (manued-get-end-point   (car  swap-pstr-list)))
	(end-gamma   nil)
	(beg-comment nil)
	(end-comment nil))
    (if (not (null comment-psr))
	(progn
	  (setq end-gamma   (manued-get-first-point comment-psr))
	  (setq beg-comment (manued-get-end-point   comment-psr))
	  (setq end-comment (manued-get-first-point end-lap)))
      (setq end-gamma (manued-get-first-point end-lap)))
    (let ((alpha  (buffer-substring beg-alpha end-alpha))
	  (beta   (buffer-substring beg-beta  end-beta))
	  (gamma  (buffer-substring beg-gamma end-gamma))
	  (comment nil))
      (if (not (null beg-comment))	; wenn es gibt ein comment.
	  (setq comment (buffer-substring beg-comment end-comment)))
      (goto-char (manued-get-first-point beg-lap))
      ;; delete manued command
      (delete-region (manued-get-first-point beg-lap) 
		     (manued-get-end-point   end-lap))
      ;; insert the result
      (let ((erst    "")
	    (zweite  "")
	    (dritter ""))
	(if manued-show-newer-p
	    (progn
	      (setq erst    gamma)
	      (setq zweite  beta)
	      (setq dritter alpha))
	  (progn
	    (setq erst    alpha)
	    (setq zweite  beta)
	    (setq dritter gamma)))
	(if (not manued-pretty-print-on-p)
	    ;; no pretty print
	    (insert (concat erst zweite dritter))
	  ;; pretty print 
	  (let ((e-str "")
		(z-str "")
		(d-str "")
		(v-str ""))
	    (setq e-str 
		  (manued-get-pretty-print-str
		   (nth 0 manued-pretty-print-format-swap-list) 
		   erst))
	    (setq z-str
		  (manued-get-pretty-print-str
		   (nth 1 manued-pretty-print-format-swap-list)
		   zweite))
	    (setq d-str
		  (manued-get-pretty-print-str
		   (nth 2 manued-pretty-print-format-swap-list) 
		   dritter))
	    (setq v-str
		  (manued-get-pretty-print-str
		   (nth 3 manued-pretty-print-format-swap-list)
		   comment))
	    (insert (concat e-str z-str d-str v-str))))
	(set-marker end-marker (point))
	(manued-proc-escape (manued-get-first-point beg-lap) (point))
	(goto-char (marker-position end-marker))
	(set-marker end-marker nil)))))
;;
;; delete $B%3%^%s%I$r=hM}$9$k(B
;;   Escape $B=hM}$N%"%k%4%j%:%`$O(B swap $B$HF1MM(B
;;
(defun manued-replace-one-delete-term (delete-pstr-list beg-lap end-lap 
							comment-psr)
  (let ((end-marker (make-marker))
	(A-part-str  nil)
	(B-part-str  nil)
	(C-part-str  nil)
	(insert-part-str  nil)
	(delete-part-str  nil)
	(comment-part-str nil))
    ;; [^A^/B;C]
    (setq A-part-str (buffer-substring 
		      (manued-get-end-point beg-lap)
		      (manued-get-first-point (car delete-pstr-list))))
    (if comment-psr 
	(progn 
	  ;; [A/^B^;C]
	  (setq B-part-str (buffer-substring
			    (manued-get-end-point (car delete-pstr-list))
			    (manued-get-first-point comment-psr)))
	  ;; [A/B;^C^]
	  (setq C-part-str (buffer-substring
			    (manued-get-end-point comment-psr)
			    (manued-get-first-point end-lap))))
      (progn
	;; [A/^B^]
	(setq B-part-str (buffer-substring
			  (manued-get-end-point (car delete-pstr-list))
			  (manued-get-first-point end-lap)))
	;; [A/B] ... keine C Teil
	(setq C-part-str nil)))
    ;; get first region?  : thanks for KAWAMURA
    ;;	true case 1 : if `show older' and `older first'
    ;;	     case 2 : if `show newer' and `newer first'
    (if (or (and (not manued-show-newer-p) manued-is-order-older-first)
	    (and  manued-show-newer-p     (not manued-is-order-older-first)))
	(progn 
	  (setq insert-part-str  A-part-str)
	  (setq delete-part-str  B-part-str)
	  (setq comment-part-str C-part-str))
      ;; or last region
      (progn 
	(setq insert-part-str  B-part-str)
	(setq delete-part-str  A-part-str)
	(setq comment-part-str C-part-str)))
      
    ;; delete manued command
    (delete-region (manued-get-first-point beg-lap) 
		   (manued-get-end-point   end-lap))

    ;; insert the result
    (if (not manued-pretty-print-on-p)
	;; no pretty print 
	(insert insert-part-str)		
      ;; do pretty print
      (let ((i-str "")
	    (d-str "")
	    (c-str ""))
	(setq i-str
	      (manued-get-pretty-print-str
	       (nth 0 manued-pretty-print-format-delete-list) 
	       insert-part-str))
	(setq d-str
	      (manued-get-pretty-print-str
	       (nth 1 manued-pretty-print-format-delete-list) 
	       delete-part-str))
	(setq c-str
	      (manued-get-pretty-print-str
	       (nth 2 manued-pretty-print-format-delete-list) 
	       comment-part-str))
	(insert (concat i-str d-str c-str))))
    (set-marker end-marker (point))
    (manued-proc-escape (manued-get-first-point beg-lap) (point))
    (goto-char (marker-position end-marker))
    (set-marker end-marker nil)))

;;
;; null $B%3%^%s%I$r=hM}$9$k(B
;;	null $B%3%^%s%I$O>C5n$7$F$7$^$C$FNI$$$@$m$&(B
;;
(defun manued-replace-one-null-term (beg-lap end-lap comment-psr)
  (let ((end-marker (make-marker)))
    (set-marker end-marker (manued-get-end-point end-lap))
    (delete-region (manued-get-first-point beg-lap)
		   (manued-get-end-point   end-lap))
    (goto-char (marker-position end-marker))
    (set-marker end-marker nil)))

;;
;; proc-escape : 
;; 	begin-point $B$+$i(B end-point $B$^$G$N4V$G(B escape $B$r=hM}$9$k(B
;;
(defun manued-proc-escape (begin-point end-point)
  "process escape 
  begin-point end-point $B$^$G$KB8:_$9$k%(%9%1!<%W$N=hM}(B"
  (goto-char end-point)				
  (let ((m-end (point-marker))		; $BJT=8Cf$KJQ2=$9$k$N$G:G8e$K0u(B
	(m-cont nil))			
    (goto-char begin-point)
    (let ((esc-pstr t))
      (while esc-pstr			; $BHO0OFb$G(B escape $BJ8;z$,$"$k8B$j(B
	(setq esc-pstr (manued-search-command
			(regexp-quote manued-escape-str)
			(marker-position m-end)))
	(if (not (null esc-pstr))
	    (progn 
	      (goto-char (manued-get-end-point esc-pstr))
	      (setq m-cont (point-marker)) ; esc $B$N:G8e$K(B mark
	      (delete-region (manued-get-first-point esc-pstr) ; del esc str
			     (manued-get-end-point   esc-pstr)) 
	      (goto-char (marker-position m-cont)) ; esc $B$ND>8e$K0\F0(B
	      (set-marker m-cont nil)
	      (forward-char 1))		; esc $B$N<!$NJ8;z$rHt$P$9(B
	  ;; else (null esc-pstr) $B$J$i=*N;(B
	  )))
    (set-marker m-end nil)))


;;
;; $B=*N;$N3g8L$,$_$D$+$i$J$$>l9g$N%(%i!<=hM}(B
;;
(defun manued-replace-can-not-find-end-paren (start-ptr)
  (setq replace-err-occur-pos (manued-get-first-point start-ptr))
  (error (format "in replace : lack of `%s' of this `%s'"
		 manued-r-parenthesis-str manued-l-parenthesis-str)))


;;============================================================
;; editting command
;;============================================================
;; insert delete command
;;   with comment or not
;;------------------------------
(defun manued-insert-delete-command-toggle-comment (b e)
  "delete command $B$NA^F~;~$K%3%a%s%HA^F~$r%G%U%)%k%H$H5U$K$9$k(B
manued-is-delete-command-with-comment-on $B$r5U$N>uBV$K$7$F(B 
manued-insert-delete-command $B$r8F$V!%(B"
  (interactive "r")
  (let ((manued-is-delete-command-with-comment-on ; dynamic binding!
	 (not manued-is-delete-command-with-comment-on)))
    (manued-insert-delete-command b e)))

;;
;; insert delete command
;;
(defun manued-insert-delete-command (b e)
  "insert delete command"
  (interactive "r")
  (let ((imark))			
    (goto-char e)			; $B8e$m$+$i=hM}$9$k(B
    (if manued-is-order-older-first	; [region/] 
	(progn				
	  (insert manued-delete-str)	; /
	  (setq imark (point-marker))	; /^
	  (if manued-is-delete-command-with-comment-on
	      (insert manued-comment-str)))) ; /^;
    (insert manued-r-parenthesis-str)	; /^;] or /^]
    (goto-char b)			; $BA0J}$X(B
    (insert manued-l-parenthesis-str)	; [
    (if (not manued-is-order-older-first) ; [/region] 
	(progn
	  (setq imark (point-marker))	; [^
	  (insert manued-delete-str)
	  (if manued-is-delete-command-with-comment-on
	      (insert manued-comment-str)))) ; [^/ or [
    (goto-char (marker-position imark))
    (set-marker imark nil)))		; for GC 

;;------------------------------
;; insert swap command
;;   with comment or not
;;------------------------------
(defun manued-insert-swap-command-toggle-comment (b e)
  "swap command $B$NA^F~;~$K%3%a%s%HA^F~$r%G%U%)%k%H$H5U$K$9$k(B
manued-is-swap-command-with-comment-on $B$r5U$N>uBV$K$7$F(B 
manued-insert-swap-command $B$r8F$V!%(B"
  (interactive "r")
  (let ((manued-is-swap-command-with-comment-on
	 (not manued-is-swap-command-with-comment-on)))
    (manued-insert-swap-command b e)))


;;
;; insert swap command
;;
(defun manued-insert-swap-command (b e)
  "insert swap command"
  (interactive "r")
  (let ((imark))			
    (goto-char e)			; $B8e$m$+$i=hM}$9$k(B [region||] 
    (insert manued-swap-str)		; |
    (setq imark (point-marker))		; |^|
    (insert manued-swap-str)		; |^|
    (if manued-is-swap-command-with-comment-on
	(insert manued-comment-str))	; ||^;
    (insert manued-r-parenthesis-str)	; ||^;] or ||^]
    (goto-char b)			; $BA0J}$X(B
    (insert manued-l-parenthesis-str)	; [
    (goto-char (marker-position imark))
    (set-marker imark nil)))		; for GC 

;;------------------------------
;; insert manued comment
;;    $B$"$kJQ?t$K$h$C$F0z?t$N0c$&4X?t$r8F$V$N$O$I$&$9$k$N$+(B
;;------------------------------
;(defun manued-insert-comment (p)
;  (interactive "d")
;  (manued-insert-comment-at-point p))
;(defun manued-insert-comment (b e)
;  (interactive "r")
;  (manued-insert-comment-region b e))

;;
;; insert manued comment
;;
(defun manued-insert-comment-at-point (p)
  "insert manued commnet at this point"
  (interactive "d")
  (goto-char p)
  (let ((imark))
    (insert manued-l-parenthesis-str)
    (insert manued-comment-str)
    (setq imark (point-marker))
    (insert manued-r-parenthesis-str)    
    (goto-char (marker-position imark))
    (set-marker imark nil)))

;;
;; insert manued comment with region 1998$BG/(B8$B7n(B11$BF|(B($B2P(B)
;;
(defun manued-insert-comment-region (b e)
  "insert manued commnet with region"
  (interactive "r")
  (let ((imark) (eq-begin-and-end (= b e)))
    (setq imark (point-marker))
    (goto-char e)
    (insert manued-r-parenthesis-str)    
    (goto-char b)
    (insert manued-l-parenthesis-str)
    (insert manued-comment-str)
    (goto-char (marker-position imark))
    ;; mark $B$7$?>l=j$KA^F~$7$?>l9g$K$O(B [; $B$NJ,$rF0$$$F$9$0F~NO$G$-$k$h$&$K$9$k(B
    (if eq-begin-and-end
	(goto-char (+ (point) 
		      (length manued-l-parenthesis-str) 
		      (length manued-comment-str))))
    (set-marker imark nil)))

;;============================================================
;; moving command
;;============================================================
(defun manued-next-l-parenthesis (p)
  "goto next manued L-parenthesis"
  (interactive "d")
  (goto-char p)
  (let ((matchpos))			; (nowpos p) deleteme
    (setq matchpos 
	  (re-search-forward 
	   (format "[^%s]%s" 
		   (regexp-quote manued-escape-str)
		   (regexp-quote manued-l-parenthesis-str)) 
	   (manued-get-doc-end-point) t))
    (if (null matchpos)			; non exist
	(progn 
	  (message "manued: no more next manued L-parenthesis.")
	  (beep))
      (backward-char 1))))

(defun manued-previous-l-parenthesis (p)
  "goto previous manued l parenthesis"  
  (interactive "d")
  (goto-char p)
  (let ((matchpos))			; (nowpos p) deleteme
    (setq matchpos 
	  (re-search-backward 
	   (format "[^%s]%s" 
		   (regexp-quote manued-escape-str)
		   (regexp-quote manued-l-parenthesis-str)) 
	   (manued-get-doc-begin-point) t))
    (if (null matchpos)			; non exist
	(progn 
	  (message "manued: no more previous manued L-parenthesis.")
	  (beep))
      (backward-char -1))))
  

(defun manued-eval-last-manuexp (p)
  "Eval last manued-command.
Evauate last manued command from now point. Same as eval-last-sexp.
When argment is setted, reciprocal mode is used. Ex. when now mode is 
show-newer, then evaluating with show-older. However this command
search backword and using first matched l-parenthesis-str, this
command mistakes interpretation for the command. 

Ex. `[A/B; [a/hello]' must be `B' in manued fashion, may evaluated to
    `[A/B; hello'

$BD>A0$N0l$D$N??s,F;%3%^%s%I$NI>2A$r9T$&!%(Beval-last-sexp $B$HF1MM!%0z?t$r(B
$BM?$($k$H8=:_$,(B newer $B$J$i$P(B older $B$r!$$H$$$&$h$&$K5U$NF0:n$r$9$k!%$?$@(B
$B$7!$8eJ}$+$i:G=i$K%^%C%A$7$?(B manued-l-parenthesis-str $B$rC5$9$N$G!$%3%a(B
$B%s%HFbIt$K??s,F;%3%^%s%I$,$"$k>l9g$K$O@5$7$/=hM}$5$l$J$$$+$b$7$l$J$$!%(B
$B$?$H$($P>e$NNc$N$h$&$J>l9g$,$"$k!%(B"
  (interactive "p")
  (save-excursion 
    (let ((is-newer))
      (if (> p 1)
	  (setq is-newer (not manued-is-now-hilit-newer))
	(setq is-newer manued-is-now-hilit-newer))
      (let ((last-manuexp-region (manued-lastexp-region)))
	(if (null last-manuexp-region)
	    (progn 
	      (error "manued-eval-last-manuexp, cannot find last manuexp."))
	  (progn
	    (goto-char (car last-manuexp-region))
	    (if is-newer
		(manued-show-newer-region (car  last-manuexp-region)
					  (car (cdr last-manuexp-region)))
	      (manued-show-older-region (car  last-manuexp-region)
					   (car (cdr last-manuexp-region))))))))))

;;
;; get last region
;;
(defun manued-lastexp-region ()
  (let ((end-pstr nil) (beg-pstr nil) (cont t))
    ;; find last ]
    (while cont
      (let ((cur-pstr (manued-search-last-command-backword-pstr)))
	(cond
	 ((manued-command-eq cur-pstr manued-r-parenthesis-str)
	  (setq end-pstr cur-pstr)
	  (setq cont nil))
	 ((manued-command-eq cur-pstr manued-l-parenthesis-str)
	  t)				; ignore
	 ((eq cur-pstr nil)
	  (error "Can not find last manued exp."))
	 (t 
	  (error "Internal error. manued-lastexp-region.1")))))
    ;; find '[' which corresponding last `]'
    (let ((paren-level 1) (cont t))
      (while cont
	(let ((cur-pstr (manued-search-last-command-backword-pstr)))
	  (cond
	   ((manued-command-eq cur-pstr manued-r-parenthesis-str)
	    (setq paren-level (1+ paren-level)))
	   ((manued-command-eq cur-pstr manued-l-parenthesis-str)
	    (setq paren-level (1- paren-level))
	    (if (= paren-level 0)
		(progn 
		  (setq beg-pstr cur-pstr)
		  (setq cont nil))))
	   ((eq cur-pstr nil)
	    (error "Can not find paren corresponding to last manued exp."))
	   (t 
	    (error "Internal error. manued-lastexp-region.2"))))))
    (list (manued-get-first-point beg-pstr) 
	  (manued-get-end-point   end-pstr))))
	
;;
;; escape $B$5$l$F$$$J$$3g8L$rC5$9(B
;;
(defun manued-lastexp-detect-pat ()
  (concat 
   (regexp-quote manued-l-parenthesis-str) 				"\\|"
   (regexp-quote manued-r-parenthesis-str)))

;;
;; escape $B$5$l$F$$$k$b$N$OC5$5$J$$(B
;;
(defun manued-lastexp-non-detect-pat ()	
  (concat 
   (regexp-quote (concat manued-escape-str manued-l-parenthesis-str))	"\\|"
   (regexp-quote (concat manued-escape-str manued-r-parenthesis-str))))

;;
;; $B8eJ}$+$iA0J}$K(B def[LR]parenthesis $B$N$_$rC5:w$9$k(B
;;
;; backword $B$G(B regex $B$KA*Br$r=q$$$F$b:GD90lCW$G$J$$$h$&$G$"$k!%%^%K%e(B
;; $B%"%k$K$h$k$H!$(Bre-search-backword $B$O(B re-search-forward $B$N40A4$J%_%i!<(B
;; $B$G$O$J$$$H$$$&$3$H$@!%$&!<$`!%(B
;;
(defun manued-search-last-command-backword-pstr ()
  (let ((ret-pstr nil) (cont t) (found nil))
    (while cont
      (if (null (re-search-backward (manued-lastexp-detect-pat) 
				    (manued-get-doc-begin-point) t))
	  (setq cont nil)		; re-search-backward $B$K<:GT(B
	(progn 
	  (setq ret-pstr 
		(list (match-beginning 0)
		      (match-end 0)
		      (buffer-substring (match-beginning 0) (match-end 0))))
	  (if (< 0 (- (point) (length manued-escape-str)))
	      (progn 
		(goto-char (- (point) (length manued-escape-str)))
		(if (not (looking-at (manued-lastexp-non-detect-pat)))
		    (progn
		      (setq found t)	; found
		      (setq cont nil)	
		      ;; $B8IN)3g8L$,$_$D$+$C$?$J$iLa$C$F$*$/(B
		      (goto-char (+ (point) (length manued-escape-str))))
		  ;; $B$3$N9T$G$O(B ~]$B$+(B ~[ $B$J$N$GL5;k$7$F<!$N(B while loop $B$r2s$k(B
		  ))
	    (progn 
	      (setq found t)
	      (setq cont nil))))))	; $B$3$l0J>eA0$K$O$$$1$J$$$N$GH/8+(B
    (if found 
	ret-pstr			;$BH/8+$7$F$$$k>l9g(B
      nil)))
	
;;============================================================
;; initialize manued 
;;	initialize variables and get defcommands.
;;============================================================
(defun manued-init-vars ()
  "Initialize variables.
	search and get document begin point and end point.
	$B??s,F;J8=qCf$+$iI,MW$J>pJs$r<h$j=P$7!$=i4|2=$9$k!%(B"
  (save-excursion
    ;;(setq manued-doc-begin-point nil)
    ;;(setq manued-doc-end-point   nil)
    (manued-get-doc-begin-point)
    (manued-get-doc-end-point)
    ;; defcommand$B$rC5$7!$CM$r%;%C%H$9$k(B
    (manued-search-set-defcommands)
    ;; color mode $B$,;H$($J$$$h$&$J$i(B off $B$K$9$k(B
    (if (not window-system)
	;; not window-system 
	(progn
	  (if manued-use-color-hilit	; $B$3$N;~$O(Bhilit$B$O6/@)E*$K(Boff
	      (progn 
		(message 
		 "manued: This window may not be able to use color. Color mode is off.")
		(setq manued-use-color-hilit nil)))))
    ;; re-initialize color face
    ))

;;
;; if use-color 'follow-font-lock-mode, according to font-lock-mode 
;;   @return when t use color, nil not use
;;
(defun manued-guess-color-mode ()
  (cond ((eq manued-use-color-hilit 'follow-font-lock-mode)
	 (cond (manued-xemacs-p font-lock-mode)		; xemacs
	       (t			; mule 
		;;(if (boundp 'font-lock-mode)
		;; font-lock-mode	; value
		;;  nil))))
		;; mule, emacs20 $B$G$O(Bfont-lock-mode$B$H6&B8$G$-$F$$$J$$!%(B
		t)))
	(t manued-use-color-hilit)))	; follow $B$G$J$1$l$P$3$NCM$rJV$9(B

;;============================================================
;; manued-minor-mode
;;	Manued has the minor-mode only.
;;============================================================
;; menu-bar
;;	$B??s,F;%a%K%e!<(B
;;	ref. mew.el 	 by Kazu Yamamoto <Kazu@Mew.org>
;;	     easymenu.el by rms.
;;------------------------------------------------------------
(if (or manued-xemacs-p (string< "20" emacs-version)) (require 'easymenu))
(defvar manued-mode-menu-bar-map nil)
(defconst manued-minor-mode-menu-spec
  '("Manued"
    ["Hilit revised  part"	manued-hilit-newer	t]    
    ["Hilit original part"	manued-hilit-older	t]
    "---"
    ("Extract document"
     ["Get revised  document in manued buf"
      manued-show-newer-in-manued-buffer t]
     ["Get original document in manued buf"	
      manued-show-older-in-manued-buffer t]
     ["Get revised  document from region"	
      manued-show-newer-region	t]
     ["Get original document from region"	
      manued-show-older-region	t]
     ["set pretty print on"	
      manued-set-pretty-print-on t]
     ["set pretty print off"	
      manued-set-pretty-print-off t])
    ["Eval last manuedexp"		manued-eval-last-manuexp t]
    "---"
    ("Insert command"
     ["Swap    region"               	manued-insert-swap-command t]
     ["Delete  region"               	manued-insert-delete-command t]
     ["Comment region"              	manued-insert-comment-region t]
     ["Header"              		manued-insert-header	t])
    "---"    
    ["Search next"              	manued-next-l-parenthesis t]
    ["Search previous"          	manued-previous-l-parenthesis t]
    ["Show version"          		manued-show-version t]))


;;------------------------------
;; $B??s,F;$N(B minor mode keymap
;;------------------------------
(defvar manued-minor-mode-map nil 
  "* keymap of manued. minor mode.
   $B??s,F;$N%-!<%^%C%W(B:
\\<manued-minor-mode-map>")

(if (null manued-minor-mode-map)
    (progn
      (setq manued-minor-mode-map (make-sparse-keymap))
      ;;(define-key manued-minor-mode-map 
      ;; manued-mode-menu-bar-map) deleteme

      (define-key manued-minor-mode-map "\C-c\C-m\C-d"
	'manued-insert-delete-command) 
      (define-key manued-minor-mode-map "\C-c\C-md"
	'manued-insert-delete-command-toggle-comment) 

      (define-key manued-minor-mode-map "\C-c\C-m\C-s"
	'manued-insert-swap-command) 
      (define-key manued-minor-mode-map "\C-c\C-ms"    
	'manued-insert-swap-command-toggle-comment) 

      (define-key manued-minor-mode-map "\C-l"     
	'manued-recenter-hilit)

      (define-key manued-minor-mode-map "\C-c\C-m\C-c" 
	'manued-insert-comment-region)

      (define-key manued-minor-mode-map "\M-n"     
	'manued-next-l-parenthesis)
      (define-key manued-minor-mode-map "\M-p"     
	'manued-previous-l-parenthesis)

      (define-key manued-minor-mode-map "\C-c\C-m\C-e"
	'manued-eval-last-manuexp)
      ;; other definitions

      ;; 
      (easy-menu-define
       manued-minor-mode-menu
       manued-minor-mode-map
       "Manued : a proofreading method."
       manued-minor-mode-menu-spec)))


;;------------------------------------------------------------
;; syntax, abbrev tables
;;------------------------------------------------------------
(defvar manued-mode-syntax-table text-mode-syntax-table
  "* syntax table of manued mode : default is `text-mode-syntax-table'
$B??s,F;%b!<%I$N(B syntax table : default $BCM$O(B `text-mode-syntax-table'")

(defvar manued-mode-abbrev-table text-mode-abbrev-table
  "* abbrev table of manued mode : default is `text-mode-abbrev-table'
$B??s,F;%b!<%I$N(B abbrev table : default $BCM$O(B `text-mode-abbrev-table'")

;;------------------------------------------------------------
;; $B2a5n$N%P!<%8%g%s$H$N:9J,$r5[<}$9$k(B
;;	$B$7$+$7:#$OBG$D<j$,$J$$$J(B 
;;	$B%X%C%@$,$J$$$N$+%P!<%8%g%s$,8E$$$N$+$O$I$&$7$h$&$b$J$$5$$,$9$k(B
;;------------------------------------------------------------
(defun manued-dispatch-for-old-version ()
  (progn
    ;; swap-str $B$r(B delete-str $B$H4V0c$($F$$$?%P!<%8%g%sMQ(B
    ;; $B$3$3$,(B string= $B$G$J$/(B match $B$J$N$OItJ,E*$KF1$8>l9g$KLdBj$,H/@8(B
    ;; $B$9$k$+$H9M$($?$?$a(B ex. defparentheses ( ), defswap )| $B$N>l9g(B
    ;; (a/b)$B$N<!$K(B | $B$,J?J8Cf$K=P$?>l9g$K:$$k!%$H;W$&$,;n$7$F$_$h$&(B
    (if (string-match manued-swap-str manued-delete-str)
    ;; (if (string-equal manued-swap-str manued-delete-str)
	(if (y-or-n-p 
	     "Buffer is edited by old manued.el. Change swap-str defs?")
	    (progn 
	      (setq manued-swap-str "|")
	      (setq manued-delete-str "/"))))
    ;; defLparenthesis, defRparenthesis $B$,B8:_$9$k>l9g$K7Y9p$9$k(B
    (goto-char (manued-get-doc-begin-point))
    (if (re-search-forward "def[LR]parenthesis" nil t)
	(progn 
	  (ding)
	  (message 
	   "def[LR]parenthesis is supported but obsolete, use defparentheses")
	  (sit-for 1)))))
;; version $BHf3S(B (string< "1.0.0" "1.0.10") $B$N>l9g$K$O<:GT$9$k!%!%!%(B

;;============================================================
;; $B??s,F;(B minor $B%b!<%I$KF~$k!%(B
;;============================================================
(defvar manued-minor-mode nil)		; $B??s,F;(B minor mode $B$K$$$k$+$I$&$+(B
(defvar manued-pushd-menubar nil)	; $B85$N(B Menubar $B$rJ]B8$9$k(B
(defalias 'manued-mode 'manued-minor-mode)
(defun manued-minor-mode (&optional arg)
  "Toggle Manued minor mode.
With arg, turn Manued minor mode on if arg is positive, off otherwise.
See the command `manued-mode' for more information on this mode.

Manued-minor-mode (In English)


emphasis original document.		\\[manued-hilit-older]
emphasis proofreaded document.		\\[manued-hilit-newer]
re-hilit				\\[manued-hilit]
show the fixed document in other buffer	\\[manued-show-older-in-manued-buffer]
show the original document in other buffer	\\[manued-show-newer-in-manued-buffer]
set revised document without pretty print	\\[manued-set-pretty-print-off]
set revised document with pretty print	\\[manued-set-pretty-print-on]
insert manued delete command		\\[manued-insert-delete-command]
insert manued swap   command		\\[manued-insert-swap-command]

insert comment command			\\[manued-insert-comment]
eval last manued command		\\[manued-eval-last-manuexp]

Manued-minor-mode (In Japanese)
$B%^%$%J!<%b!<%I(B

$B=$@5A0$NJ8=q$N%O%$%i%$%H(B		\\[manued-hilit-older]
$B=$@58e$NJ8=q$N%O%$%i%$%H(B		\\[manued-hilit-newer]
$B:F%O%$%i%$%H(B			\\[manued-hilit]
$B=$@5A0$NJ8=q$rJL$N%P%C%U%!$KI=<((B	\\[manued-show-older-in-manued-buffer]
$B=$@58e$NJ8=q$rJL$N%P%C%U%!$KI=<((B	\\[manued-show-newer-in-manued-buffer]
$B=$@5A0$NJ8=q$r(Bpretty print$B%9%?%$%k$GI=<((B	\\[manued-set-pretty-print-on]
$B=$@5A0$NJ8=q$rDL>o$N%9%?%$%k$GI=<((B	\\[manued-set-pretty-print-off]

$B??s,F;(B delete $B%3%^%s%I$NF~NO(B		\\[manued-insert-delete-command]
$B??s,F;(B swap $B%3%^%s%I$NF~NO(B		\\[manued-insert-swap-command]
$B??s,F;(B comment $B$NF~NO(B			\\[manued-insert-comment]
$BD>A0$N??s,F;%3%^%s%I$NI>2A(B	       	\\[manued-eval-last-manuexp]

Special Commands: 

\\{manued-minor-mode-map}
"

  (interactive "P")
  (make-variable-buffer-local 'manued-minor-mode)
  (setq manued-minor-mode
	(if (null arg) (not manued-minor-mode) ; toggle
	  (> (prefix-numeric-value arg) 0)))
  (if manued-minor-mode
      (progn
	(manued-init-vars)
	(run-hooks 'manued-minor-mode-hook)
	(manued-search-and-insert-header)
	(manued-add-menu-bar)		; add manued menu to manu bar 
	(manued-hilit))			; hilit
    ;; 
    (if window-system
	(progn
	  (manued-delele-menu-bar)	  ; and menu bar
	  ))))

;;
;; add menu bar
;; $B$I$&$d$i!$(Bxemacs$B$G$J$$(BEmacs$B$G$O(Beasymenu$B$,(Badd, delete$B$^$G$_$F$/$l$k$h$&$@(B
;;
(defun manued-add-menu-bar ()
  (if manued-xemacs-p
      (progn 
	(setq manued-pushd-menubar current-menubar)
	(set-buffer-menubar current-menubar) ; for buffer local 
	(add-submenu nil manued-minor-mode-menu-spec))
    ;; nothing to do in non xemacs enviroment
    ()					
    ))

;;
;; delete menu bar
;;
(defun manued-delele-menu-bar ()
  (if manued-xemacs-p
      (set-buffer-menubar manued-pushd-menubar)
    ;; nothing to do in non xemacs enviroment
    ()
    ))

;;
;; minor mode alist
;;
(or (assq 'manued-minor-mode minor-mode-alist)
    (setq minor-mode-alist
	  (cons '(manued-minor-mode " Manued") minor-mode-alist)))

;;
;; minor mode keymap alist 
;;
(or (assq 'manued-minor-mode minor-mode-map-alist)
    (setq minor-mode-map-alist
	  (cons (cons 'manued-minor-mode manued-minor-mode-map)
		minor-mode-map-alist)))
;;
(provide 'manued)
;; end of manued.el ------------------------------
